import 'dart:io';

import 'package:fast_flutter_driver_tool/src/preparing_tests/test_generator/test_generator.dart';
import 'package:file/memory.dart';
import 'package:mockito/mockito.dart';
import 'package:test/test.dart';

void main() {
  group('testFiles', () {
    test('list test file', () {
      const testName = 'my_test.dart';
      final mockDir = _MockDirectory();
      when(mockDir.listSync(recursive: anyNamed('recursive')))
          .thenReturn([File(testName)]);

      final tested = testFiles(mockDir, 'any');

      expect(tested, contains(testName));
    });

    test('skips non test files', () {
      const testName = 'my.dart';
      final mockDir = _MockDirectory();
      when(mockDir.listSync(recursive: anyNamed('recursive')))
          .thenReturn([File(testName)]);

      final tested = testFiles(mockDir, 'any');

      expect(tested, isEmpty);
    });

    test('skips excluded file', () {
      const testName = 'my_test.dart';
      final mockDir = _MockDirectory();
      when(mockDir.listSync(recursive: anyNamed('recursive')))
          .thenReturn([File(testName)]);

      final tested = testFiles(mockDir, testName);

      expect(tested, isEmpty);
    });
  });

  group('generateTestFile', () {
    File aggregatedFile;
    Directory mockDir;
    setUp(() {
      aggregatedFile = MemoryFileSystem().file('test.dart');
      mockDir = _MockDirectory();
      when(mockDir.path).thenReturn('/');
      when(mockDir.listSync(recursive: anyNamed('recursive'))).thenReturn([
        File('my_test.dart'),
      ]);
    });
    test('writes aggregated test file with args', () async {
      const content = '''
// ignore_for_file: directives_ordering
/// This file is autogenerated and should not be committed to source control

import 'my_test.dart' as my;

void main(List<String> args) {
  my.main(args);
}
''';

      await generateTestFile(aggregatedFile, mockDir, '', hasArguments: true);

      expect(aggregatedFile.readAsStringSync(), content);
    });

    test('writes aggregated test file without args', () async {
      const content = '''
// ignore_for_file: directives_ordering
/// This file is autogenerated and should not be committed to source control

import 'my_test.dart' as my;

void main() {
  my.main();
}
''';

      await generateTestFile(aggregatedFile, mockDir, '', hasArguments: false);

      expect(aggregatedFile.readAsStringSync(), content);
    });
  });
}

class _MockDirectory extends Mock implements Directory {}
